package dk.brics.xact;

import dk.brics.misc.Origin;

/**
 * Attribute node.
 */
public class Attribute extends AttrNode {

	private final String value;

	/**
	 * Constructs a new attribute node.
	 */
	public Attribute(String namespace, String localname, String value, AttrNode nextattribute, Origin origin) {
		super(namespace, localname, nextattribute, origin);
		if (!StringTypes.isChars(value))
			throw new XMLWellformednessException("illegal attribute value");
		this.value = value;
	}

	/**
	 * Constructs a new attribute node.
	 */
	public Attribute(String namespace, String localname, String value) {
		this(namespace, localname, value, null, null);
	}

	/**
	 * Constructs a new attribute node with no namespace.
	 */
	public Attribute(String localname, String value) {
		this(null, localname, value, null, null);
	}

	/**
	 * Constructs a new attribute node.
	 */
	public Attribute(String namespace, String localname, String value, Origin origin) {
		this(namespace, localname, value, null, origin);
	}

	/**
	 * Constructs a new attribute node with no namespace.
	 */
	public Attribute(String localname, String value, Origin origin) {
		this(null, localname, value, null, origin);
	}

	/**
	 * Returns the value.
	 */
	public String getValue() {
		return value;
	}

	@Override
	void visitNormalizedBy(NodeVisitor v) {
		v.visit(this);
	}
	
	@Override
	<R,A> R visitNormalizedBy(NodeVisitorParameterized<R,A> v, A arg) {
		return v.visit(this, arg);
	}
	
	@Override
	void visitAnyBy(AnyNodeVisitor v) {
		v.visit(this);
	}

	@Override
	public Attribute copy(AttrNode nextattribute) {
		return new Attribute(getNamespace(), getLocalName(), value, nextattribute, getOrigin());
	}
	
	@Override
	public String toString() {
		return "[Attribute " + expanded(getNamespace(), getLocalName()) + " " + value + "]";
	}
}
