package dk.brics.xact;

import dk.brics.misc.Origin;

/**
 * Attribute gap node.
 */
public class AttributeGap extends AttrNode {
	
	private final String gap;

	private final String type;

	/**
	 * Constructs a new attribute gap node.
	 */
	public AttributeGap(String namespace, String localname, String gap, String type, AttrNode nextattribute, Origin origin) {
		super(namespace, localname, nextattribute, origin);
		this.gap = gap;
		this.type = type;
	}

	/**
	 * Constructs a new attribute gap node.
	 */
	public AttributeGap(String namespace, String localname, String gap, String type) {
		this(namespace, localname, gap, type, null, null);
	}

	/**
	 * Constructs a new attribute gap node with no namespace.
	 */
	public AttributeGap(String localname, String gap, String type) {
		this(null, localname, gap, type, null, null);
	}

	/**
	 * Constructs a new attribute gap node with no namespace and no type.
	 */
	public AttributeGap(String localname, String gap) {
		this(null, localname, gap, null, null, null);
	}

	/**
	 * Constructs a new attribute gap node.
	 */
	public AttributeGap(String namespace, String localname, String gap, String type, Origin origin) {
		this(namespace, localname, gap, type, null, origin);
	}

	/**
	 * Constructs a new attribute gap node with no namespace.
	 */
	public AttributeGap(String localname, String gap, String type, Origin origin) {
		this(null, localname, gap, type, null, origin);
	}

	/**
	 * Constructs a new attribute gap node with no namespace and no type.
	 */
	public AttributeGap(String localname, String gap, Origin origin) {
		this(null, localname, gap, null, null, origin);
	}

	/**
	 * Returns the gap name.
	 */
	public String getGap() {
		return gap;
	}

	/**
	 * Returns the gap type, or null if absent.
	 */
	public String getType() {
		return type;
	}

	@Override
	void visitNormalizedBy(NodeVisitor v) {
		v.visit(this);
	}
	
	@Override
	<R,A> R visitNormalizedBy(NodeVisitorParameterized<R,A> v, A arg) {
		return v.visit(this, arg);
	}
	
	@Override
	void visitAnyBy(AnyNodeVisitor v) {
		v.visit(this);
	}

	@Override
	public AttributeGap copy(AttrNode nextattribute) {
		return new AttributeGap(getNamespace(), getLocalName(), gap, type, nextattribute, getOrigin());
	}
	
	@Override
	public String toString() {
		return "[AttributeGap " + (type != null ? type + " " : "")
		+ expanded(getNamespace(), getLocalName()) + " " + gap + " " + "]";
	}
}
