package dk.brics.xact;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * List of nodes.
 */
public class NodeList<T extends Node> implements Iterable<T> {

	private final ArrayList<T> ns;

	/**
	 * Constructs a new <code>NodeList</code>.
	 * @see XML#get(String)
	 */
	public NodeList(Iterable<T> ns) {
		this.ns = new ArrayList<T>();
		for (T n : ns)
			this.ns.add(n);
	}

	/**
	 * Concatenates the list of nodes into an XML template.
	 */
	public XML concat() {
		return XML.concat(ns);
	}  

	/**
	 * Concatenates the list of nodes into an XML template.
	 */
	public XML toXML() {
		return concat();
	}

	/**
	 * Returns a list iterator.
	 */
	public Iterator<T> iterator() {
		return ns.iterator();
	}
	
	/**
	 * Returns the node at the given index.
	 */
	public T get(int index) {
		return ns.get(index);
	}
	
	/**
	 * Returns the size of the list.
	 */
	public int size() {
		return ns.size();
	}

	/**
	 * Checks whether the list is empty.
	 */
	public boolean isEmpty() {
		return ns.isEmpty();
	}
	
	/**
	 * Visits each node in the list by the given visitor.
	 */
	public final void visitBy(NodeVisitor v) {
		for (Node n : this)
			n.visitBy(v);
	}
	
	/**
	 * Returns a description of the nodes in this list.
	 */
	@Override
	public String toString() {
		return ns.toString();
	}
}
