package dk.brics.xact;

import dk.brics.misc.Origin;

/**
 * Base class for operation nodes.
 * Used for lazily executed operations.
 */
abstract class OperationNode extends XML {

	/**
	 * Constructs a new operation node.
	 */
	OperationNode(Origin origin) {
		super(origin);
	}

	@Override
	void visitNormalizedBy(NodeVisitor v) {
		throw new RuntimeException(); // unreachable
	}
	
	@Override
	<R,A> R visitNormalizedBy(NodeVisitorParameterized<R,A> v, A arg) {
		throw new RuntimeException(); // unreachable
	}

	@Override
	boolean isTemplate() {
		return normalize(this).isTemplate();
	}
	
	@Override
	public TempNode asTemplate() {
		return (TempNode)follow(normalize(this));
	}

	@Override
	public boolean isElement() {
		return normalize(this).isElement();
	}
	
	@Override
	public Element asElement() {
		return (Element)follow(normalize(this));
	}

	@Override
	public boolean isText() {
		return normalize(this).isText();
	}

	@Override
	public Text asText() {
		return (Text)follow(normalize(this));
	}

	@Override
	public boolean isComment() {
		return normalize(this).isComment();
	}

	@Override
	public Comment asComment() {
		return (Comment)follow(normalize(this));
	}

	@Override
	public boolean isProcessingInstruction() {
		return normalize(this).isProcessingInstruction();
	}

	@Override
	public ProcessingInstruction asProcessingInstruction() {
		return (ProcessingInstruction)follow(normalize(this));
	}
}
