package dk.brics.xact;

import dk.brics.automaton.Datatypes;
import dk.brics.automaton.RunAutomaton;

/**
 * Patterns for various XML strings. 
 */
public abstract class StringTypes {
	
	private static final RunAutomaton name_automaton = new RunAutomaton(Datatypes.get("Name2"));
	
	private StringTypes() {}
	
	/**
	 * Checks whether the given string matches 
	 * <tt><a href="http://www.w3.org/TR/REC-xml/#NT-S" target="_top">S</a></tt>.
	 */
	public static boolean isWhitespace(String text) {
		if (text.length() == 0)
			return false;
		for (int i = 0; i < text.length(); i++) {
			char c = text.charAt(i);
			if (" \t\n\r".indexOf(c) == -1)
				return false;
		}
		return true;
	}

	/**
	 * Checks whether the given string matches 
	 * <tt>(<a href="http://www.w3.org/TR/REC-xml/#NT-Char" target="_top">Char</a>)*</tt>.
	 */
	public static boolean isChars(String text) {
		for (int i = 0; i < text.length(); i++) {
			char c = text.charAt(i);
			if ('\uD800' <= c && c <= '\uDBFF') {
				if (i+1 == text.length())
					return false;
				char d = text.charAt(i + 1);
				if (!('\uDC00' <= d && d <= '\uDFFF'))
					return false;
			}
			if (" \t\n\r".indexOf(c) == -1 
					&& !('\u0020' <= c && c <= '\uD7FF')
					&& !('\ue000' <= c && c <= '\ufffd')
					&& c != ' ' && c != '\t' && c != '\r' && c != '\n')
				return false;
		}
		return true;
	}

	/**
	 * Checks whether the given string matches 
	 * <tt><a href="http://www.w3.org/TR/REC-xml/#NT-Name" target="_top">Name</a></tt>.
	 */
	public static boolean isName(String text) {
		return name_automaton.run(text);
	}

	/**
	 * Checks whether the given string matches 
	 * <tt><a href="http://www.w3.org/TR/REC-xml-names/#NT-NCName" target="_top">NCName</a></tt>.
	 */
	public static boolean isNCName(String text) {
		if (text.indexOf(':') != -1)
			return false;
		return isName(text);
	}
}
