package dk.brics.xact;

import dk.brics.misc.Origin;

/**
 * Text node.
 */
public class Text extends TempNode {
	
	/**
	 * Empty text node.
	 */
	static final Text EMPTY = new Text("");
	
	private final String text;
	
	/**
	 * Constructs a new text node.
	 */
	public Text(String text, XML nextsibling, Origin origin) {
		super(nextsibling, origin);
		this.text = text;
	}

	/**
	 * Constructs a new text node.
	 */
	public Text(String text) {
		this(text, null, null);
	}

	/**
	 * Constructs a new text node.
	 */
	public Text(String text, Origin origin) {
		this(text, null, origin);
	}

	/**
	 * Returns the text string of this text node.
	 */
	@Override
	public String getString() {
		return text;
	}

	@Override
	void visitNormalizedBy(NodeVisitor v) {
		v.visit(this);
	}
	
	@Override
	<R,A> R visitNormalizedBy(NodeVisitorParameterized<R,A> v, A arg) {
		return v.visit(this, arg);
	}
	
	@Override
	void visitAnyBy(AnyNodeVisitor v) {
		v.visit(this);
	}

	@Override
	public Text copy(XML nextsibling) {
		return new Text(text, nextsibling, getOrigin());
	}
	
	@Override
	public String toString() {
		return "[Text " + text + "]";
	}
}
