package dk.brics.xact.analysis.concurrent;

import java.util.LinkedList;

/**
 * Runs all tasks in the calling thread.
 */
public class SingleThreadedTaskRunner implements XactTaskRunner {
    private boolean running=false;
    private LinkedList<Runnable> tasks = new LinkedList<Runnable>();
    public void addTask(Runnable runnable) {
        if (running)
            throw new IllegalStateException("Cannot add tasks while running");
        tasks.add(runnable);
    }
    public void runTasks() {
        if (running) {
            throw new IllegalStateException("runTasks is already running");
        }
        running=true;
        try {
            for (Runnable task : tasks) {
                task.run();
            }
        } finally {
            tasks.clear();
            running=false;
        }
    }
    public boolean maybeMultiThreaded() {
        return false;
    }
}
