package dk.brics.xact.analysis.flowgraph;

import java.util.HashMap;
import java.util.Map;

import dk.brics.misc.Origin;
import dk.brics.xmlgraph.Node;
import dk.brics.xmlgraph.XMLGraph;

/**
 * Flow graph.
 * <p>
 * Nodes correspond to abstract statements. 
 * Edges correspond to potential control-flow and are labelled with 
 * the variables that are maintained from the source to the destination.
 */
public class FlowGraph extends Graph<Statement,VariableFilter> {
	
	private Map<String,Origin> schemas; // map from schema URLs to source code origins
	
	private Map<String,String> namespaces; // map from prefixes to namespace URIs
	
	private XMLGraph xmlgraph;
	private Map<String,Node> typemap = new HashMap<String,Node>();
	
	/** 
	 * Constructs an empty graph.
	 */
	public FlowGraph(Map<String,Origin> schemas, Map<String,String> namespaces) {
		super();
		this.schemas = schemas;
		this.namespaces = namespaces;
	}
	
	/**
	 * Returns map from schema URLs to origins.
	 */
	public Map<String,Origin> getSchemas() {
		return schemas;
	}
	
	/**
	 * Returns map from namespace prefixes to namespace URIs.
	 */
	public Map<String,String> getNamespaces() {
		return namespaces;
	}
	
	public XMLGraph getXMLGraph() {
        return xmlgraph;
    }

	public void setXMLGraph(XMLGraph xmlgraph) {
        this.xmlgraph = xmlgraph;
    }
	
	public Map<String, Node> getTypemap() {
        return typemap;
    }
	
	public void setTypemap(Map<String, Node> typemap) {
        this.typemap = typemap;
    }
}
