package dk.brics.xact.analysis.flowgraph;


/** 
 * A program variable, used in a flow graph. 
 */
public class Variable {
	
	/** 
	 * Variable ID number.
	 * All variables in a graph of abstract statements will usually have unique IDs.
	 */
	private int id;
	
	/**
	 * Set if this is a field (or global dummy) variable.
	 */
	private boolean global;
	
	/** 
	 * Constructs a new variable object with no type annotation.
	 */
	public Variable(int id, boolean global) {
		this.id = id;
		this.global = global;
	}
	
	/**
	 * Returns the variable ID number.
	 */
	public int getID() {
		return id;
	}
	
	/**
	 * Returns true if this is a field (or global dummy) variable.
	 */
	public boolean isGlobal() {
		return global;
	}
	
	/**
	 * Marks this variable as being global.
	 */
	public void setGlobal() {
		this.global = true;
	}
	
	/** 
	 * Returns a name for the variable useful for printing.
	 * 'g' means "global", 'l' means "local", 'a' means "array".
	 */
	@Override
	public String toString() {
		StringBuilder b = new StringBuilder();
		if (global)
			b.append('g');
		else
			b.append('l');
		b.append(id);
		return b.toString();
	}
}
