package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.Method;
import dk.brics.xact.analysis.flowgraph.Statement;
import dk.brics.xact.analysis.flowgraph.Variable;

/**
 * A call to an application method.
 */
public class CallStm extends Statement {
	
	private Variable[] arguments;
	private Method method;
	private Variable result;
	private boolean[] mutableArguments;
	
	/**
	 * Creates a call statement. The call statement adds itself as a call site to the given method.
	 * @param result where to store the returned value
	 * @param method a method. This statement will add itself as a call site.
	 * @param arguments variables with the method's actual arguments
	 * @param origin origin
	 */
	public CallStm(Variable result, Method method, Variable[] arguments, Origin origin) {
		super(origin);
		assert method != null;
		this.result = result;
		this.method = method;
		this.arguments = arguments;
		this.mutableArguments = new boolean[arguments.length];
	}
	
	@Override
	public String getOpName() {
		return "call";
	}

	@Override
	public void visitBy(StatementVisitor v) {
		v.visitCallStm(this);
	}

	public Variable getArgument(int i) {
		return arguments[i];
	}
	public void setArgument(int i, Variable var) {
		arguments[i] = var;
	}
	public Variable[] getArguments() {
		return arguments;
	}

	public Method getMethod() {
		return method;
	}

	public void setMethod(Method method) {
		this.method = method;
	}

	public Variable getResult() {
		return result;
	}

	public void setResult(Variable result) {
		this.result = result;
	}
	
	public boolean isArgumentMutable(int i) {
		return mutableArguments[i];
	}
	public void setArgumentMutable(int i, boolean mutable) {
		mutableArguments[i] = mutable;
	}
	
	@Override
	public String toString() {
		StringBuffer b = new StringBuffer();
		b.append("call ");
		b.append(method.getName());
		b.append("(");
		for (int i=0; i<arguments.length; i++) {
			if (i > 0)
				b.append(",");
			b.append(arguments[i].toString());
		}
		b.append(") at ");
		b.append(super.toString());
		return b.toString();
	}
	
}
