package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.Variable;

/**
 * Assignment of {@link dk.brics.xact.XML#asElement()} and variants.
 */
public class CastStm extends InstanceAssignment {
	
	/**
	 * Node kinds.
	 */
	public enum Kind {ASATTRIBUTE, ASCOMMENT, ASELEMENT, ASNSDECL, ASPI, ASTEXT};
	
	private Kind kind;

	/**
	 * Constructs a new <code>CastStm</code> object.
	 */
	public CastStm(Kind kind, Variable dest, Variable base, Origin origin) {
		super(dest, base, origin);
		this.kind = kind;
	}
	
	/**
	 * Returns the kind.
	 */
	public Kind getKind() {
		return kind;
	}
	
	@Override
	public void visitBy(StatementVisitor v) { 
		v.visitCastStm(this); 
	}
	
	@Override
	public String toString() {
		return "cast[" + kind + "](" + getBase() + ")" + super.toString();
	}
	
	@Override
	public String getOpName() {
		switch (kind) {
		case ASELEMENT:
			return "asElement";
		case ASATTRIBUTE:
			return "asAttribute";
		case ASTEXT:
			return "asText";
		case ASCOMMENT:
			return "asComment";
		case ASNSDECL:
			return "asNamespaceDecl";
		case ASPI:
			return "asProcessingInstruction";
		default:
			throw new RuntimeException("Unexpected statement kind " + kind);
		}
	}
}
