package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.automaton.Automaton;
import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.Variable;

/**
 * Assignment of {@link dk.brics.xact.Element#concat(Object...)} and variants.
 */
public class ConcatStm extends Assignment {
	
	/** 
	 * The XML variable containing the XML values being concatenated.
	 */
	private Variable xmlsrc;
	
	/**
	 * The automaton containing the string values being concatenated.
	 */
	private Automaton stringsrc;
	
	/** 
	 * Constructs a new <code>ConcatStm</code> object.
	 */
	public ConcatStm(Variable dest, Automaton stringsrc, Variable xmlsrc, Origin origin) {
		super(dest, origin);
		this.stringsrc = stringsrc;
		this.xmlsrc = xmlsrc;
	}

	/**
	 * Returns the string source automaton.
	 */
	public Automaton getStringSource() {
		return stringsrc;
	}
	/**
	 * Changes the string source automaton.
	 */
	public void setStringSource(Automaton stringsrc) {
		assert stringsrc != null;
		this.stringsrc = stringsrc;
	}
	
	/**
	 * Returns the XML source variable.
	 */
	public Variable getXMLSource() {
		return xmlsrc;
	}
	
	/**
	 * Sets the XML source variable.
	 */
	public void setXMLSource(Variable xmlsrc) {
		this.xmlsrc = xmlsrc;
	}
	
	@Override
	public void visitBy(StatementVisitor v) { 
		v.visitConcatStm(this); 
	}
	
	@Override
	public String toString() {
		return "concat(" + xmlsrc + ";" + stringsrc.getNumberOfStates() + "states)" + super.toString();
	}
	
	@Override
	public String getOpName() {
		return "concat";
	}
}
