package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.Variable;

/**
 * Assignment of {@link dk.brics.xact.Element#copy(AttrNode, XML, XML)} and variants.
 */
public class CopyStm extends InstanceAssignment {
	
	public enum Kind {ELEMENT, ATTRIBUTE, ATTRIBUTEGAP, COMMENT, PROCESSINGINSTRUCTION, TEMPLATEGAP, TEXT, TEMPNODE, ATTRNODE};
	
	private Kind kind;

	/**
	 * First child, or null if none.
	 */
	private Variable firstchild;
	
	/**
	 * First attribute or attribute gap, or null if none.
	 */
	private Variable firstattr;
	
	/**
	 * Next sibling or attribute, or null if none.
	 */
	private Variable nextnode;
	
	/** 
	 * Constructs a new <code>CopyStm</code> object.
	 * <code>firstchild</code>, <code>firstattr</code> and <code>nextnode</code> can be null.
	 */
	public CopyStm(Kind kind, Variable dest, Variable base, Variable firstchild, Variable firstattr, Variable nextnode, Origin origin) {
		super(dest, base, origin);
		this.kind = kind;
		this.firstchild = firstchild;
		this.firstattr = firstattr;
		this.nextnode = nextnode;
	}

	/**
	 * Returns the kind.
	 */
	public Kind getKind() {
		return kind;
	}
	
	/**
	 * Returns the first-child variable, or null if none is available.
	 */
	public Variable getFirstChild() {
		return firstchild;
	}
	
	/**
	 * Sets the first-child variable.
	 */
	public void setFirstChild(Variable firstchild) {
		this.firstchild = firstchild;
	}
	
	/**
	 * Returns the first-attr variable, or null if none is available.
	 */
	public Variable getFirstAttr() {
		return firstattr;
	}
	
	/**
	 * Sets the first-attr variable.
	 */
	public void setFirstAttr(Variable firstattr) {
		this.firstattr = firstattr;
	}
	
	/**
	 * Returns the next-node variable, or null if none is available.
	 */
	public Variable getNextNode() {
		return nextnode;
	}
	
	/**
	 * Sets the next-node variable.
	 */
	public void setNextNode(Variable nextnode) {
		this.nextnode = nextnode;
	}
	
	@Override
	public void visitBy(StatementVisitor v) { 
		v.visitCopyStm(this); 
	}
	
	@Override
	public String toString() {
		return "copy[" + kind + "](" 
		+ getBase() 
		+ (firstchild!= null ? "," + firstchild : "") 
		+ (firstattr!= null ? "," + firstattr : "") 
		+ nextnode 
		+ ")" + super.toString();
	}
	
	@Override
	public String getOpName() {
		return "copy";
	}
}
