package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.Variable;

/**
 * Assignment of {@link dk.brics.xact.XML#get(String)} and variants.
 */
public class GetStm extends InstanceAssignment {
	
	/**
	 * Node kinds.
	 */
	public enum Kind {GET, GETELEMENT, GETELEMENTS, GETFIRSTELEMENT, GETNEXTSIBLING, GETFIRSTATTR, 
		GETFIRSTATTRIBUTE, GETFIRSTCHILD, GETNEXTATTR, GETNEXTATTRIBUTE};

	private Kind kind;
	
	/** 
	 * The XPath expression, or null if none.
	 */
	private String xpath;
	
	/** 
	 * Constructs a new <code>GetStm</code> object.
	 * <code>xpath</code> can be null.
	 */
	public GetStm(Kind kind, Variable dest, Variable base, String xpath, Origin origin) {
		super(dest, base, origin);
		this.kind = kind;
		this.xpath = xpath;
	}

	/**
	 * Returns the XPath expression.
	 */
	public String getXPath() {
		return xpath;
	}
	
	/**
	 * Returns the kind.
	 */
	public Kind getKind() {
		return kind;
	}
	
	@Override
	public void visitBy(StatementVisitor v) { 
		v.visitGetStm(this); 
	}
	
	@Override
	public String toString() {
		return "get[" + kind + "](" + getBase() + (xpath!=null ? "," + xpath : "") + ")" + super.toString();
	}
	
	@Override
	public String getOpName() {
		switch (kind) {
		case GET:
			return "get";
		case GETELEMENT:
			return "getElement";
		case GETELEMENTS:
			return "getElements";
		case GETFIRSTATTR:
			return "getFirstAttr";
		case GETFIRSTATTRIBUTE:
			return "getFirstAttribute";
		case GETFIRSTCHILD:
			return "getFirstChild";
		case GETFIRSTELEMENT:
			return "getFirstElement";
		case GETNEXTATTR:
			return "getNextAttr";
		case GETNEXTATTRIBUTE:
			return "getNextAttribute";
		case GETNEXTSIBLING:
			return "getNextSibling";
		default:
			throw new RuntimeException("Unexpected statement kind " + kind);
		}
	}
}
