package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.automaton.Automaton;
import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.Variable;

/**
 * Assignment of {@link dk.brics.xact.XML#plug(String, Object)} and variants.
 */
public class PlugStm extends InstanceAssignment {
	
	/**
	 * Node kinds.
	 */
	public enum Kind {PLUG, PLUGMULTI, CLOSE, PLUGWRAP};
	
	private Kind kind;

	/** 
	 * The name of the gap being plugged into.
	 */
	private String gapname;
	
	/** 
	 * The XML variable containing the XML values being plugged in.
	 */
	private Variable xmlsrc;
	
	/**
	 * The automaton containing the string values being plugged in.
	 */
	private Automaton stringsrc;
	
	/** 
	 * Constructs a new <code>PlugStm</code> object.
	 * <code>gapname</code>, <code>stringsrc</code> and <code>xmlsrc</code> can be null.
	 */
	public PlugStm(Kind kind, Variable dest, Variable base, String gapname, Automaton stringsrc, Variable xmlsrc, Origin origin) {
		super(dest, base, origin);
		this.kind = kind;
		this.gapname = gapname;
		this.stringsrc = stringsrc;
		this.xmlsrc = xmlsrc;
	}

	/**
	 * Returns the kind.
	 */
	public Kind getKind() {
		return kind;
	}
	
	/**
	 * Returns the string source automaton.
	 */
	public Automaton getStringSource() {
		return stringsrc;
	}
	
	
	
	public void setStringSource(Automaton stringsrc) {
		assert stringsrc != null;
		this.stringsrc = stringsrc;
	}

	/**
	 * Returns the XML source variable.
	 */
	public Variable getXMLSource() {
		return xmlsrc;
	}
	
	/**
	 * Sets the XML source variable.
	 */
	public void setXMLSource(Variable xmlsrc) {
		this.xmlsrc = xmlsrc;
	}
	
	/**
	 * Returns the gap name.
	 */
	public String getGapName() {
		return gapname;
	}
	
	@Override
	public void visitBy(StatementVisitor v) { 
		v.visitPlugStm(this); 
	}
	
	@Override
	public String toString() {
		return "plug[" + kind + "](" + getBase() 
			+ (kind != Kind.CLOSE ? "," + gapname + "," + xmlsrc + ";" + stringsrc.getNumberOfStates() + "states" : "") + ")" 
			+ super.toString();
	}
	
	@Override
	public String getOpName() {
		switch (kind) {
		case PLUGWRAP:
			return "plugWrap";
		case CLOSE:
			return "close";
		case PLUG:
		case PLUGMULTI:
			return "plug";
		default:
			throw new RuntimeException("Unexpected statement kind " + kind);
		}
	}
}
