package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.automaton.Automaton;
import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.Variable;

/**
 * Assignment of {@link dk.brics.xact.XML#set(String, Object)} and variants.
 */
public class SetStm extends InstanceAssignment {
	
	/**
	 * Node kinds.
	 */
	public enum Kind {SET, SETCONTENT};

	private Kind kind;
	
	/** 
	 * The XPath expression, or null if none.
	 */
	private String xpath;
	
	/** 
	 * The XML variable containing the XML values being inserted.
	 */
	private Variable xmlsrc;
	
	/**
	 * The automaton containing the string values being plugged in.
	 */
	private Automaton stringsrc;
	
	/** 
	 * Constructs a new <code>SetStm</code> object.
	 * <code>xpath</code> can be null.
	 */
	public SetStm(Kind kind, Variable dest, Variable base, String xpath, Automaton stringsrc, Variable xmlsrc, Origin origin) {
		super(dest, base, origin);
		this.kind = kind;
		this.xpath = xpath;
		this.stringsrc = stringsrc;
		this.xmlsrc = xmlsrc;
	}

	/**
	 * Returns the string source automaton.
	 */
	public Automaton getStringSource() {
		return stringsrc;
	}
	public void setStringSource(Automaton stringsrc) {
		this.stringsrc = stringsrc;
	}

	/**
	 * Returns the XML source variable.
	 */
	public Variable getXMLSource() {
		return xmlsrc;
	}
	
	/**
	 * Sets the XML source variable.
	 */
	public void setXMLSource(Variable source) {
		this.xmlsrc = source;
	}
	
	/**
	 * Returns the XPath expression.
	 */
	public String getXPath() {
		return xpath;
	}
	
	/**
	 * Returns the kind.
	 */
	public Kind getKind() {
		return kind;
	}
	
	@Override
	public void visitBy(StatementVisitor v) { 
		v.visitSetStm(this); 
	}
	
	@Override
	public String toString() {
		return "set[" + kind + "](" + getBase() + (xpath!=null ? "," + xpath : "") + "," + xmlsrc 
			+ ";" + automatonDescription() + ")" + super.toString();
	}
	private String automatonDescription() {
		if (stringsrc.isTotal())
			return "<all strings>";
		if (stringsrc.isEmpty())
			return "<empty>";
		if (stringsrc.isEmptyString())
			return "<empty string>";
		return stringsrc.getNumberOfStates() + "states";
	}
	
	@Override
	public String getOpName() {
		switch (kind) {
		case SET:
			return "set";
		case SETCONTENT:
			return "setContent";
		default:
			throw new RuntimeException("Unexpected statement kind " + kind);
		}	
	}
}
