package dk.brics.xact.analysis.flowgraph.statements;


/** 
 * An interface for visitors of abstract statements.
 */
public interface StatementVisitor {

	/**
	 * Visits an {@link EmptyStm}.
	 */
	public void visitEmptyStm(EmptyStm s);
	
	/**
	 * Visits an {@link UnknownStm}.
	 */
	public void visitUnknownStm(UnknownStm s);
	
	/**
	 * Visits a {@link VarStm}.
	 */
	public void visitVarStm(VarStm s);
	
	/**
	 * Visits a {@link ConstStm}.
	 */
	public void visitConstStm(ConstStm s);
	
	/**
	 * Visits a {@link GetStm}.
	 */
	public void visitGetStm(GetStm s);
	
	/**
	 * Visits a {@link CopyStm}.
	 */
	public void visitCopyStm(CopyStm s);
	
	/**
	 * Visits a {@link CastStm}.
	 */
	public void visitCastStm(CastStm s);
	
	/**
	 * Visits a {@link CheckStm}.
	 */
	public void visitCheckStm(CheckStm s);
	
	/**
	 * Visits a {@link ConcatStm}.
	 */
	public void visitConcatStm(ConcatStm s);
	
	/**
	 * Visits a {@link SetStm}.
	 */
	public void visitSetStm(SetStm s);
	
	/**
	 * Visits a {@link RemoveStm}.
	 */
	public void visitRemoveStm(RemoveStm s);
	
	/**
	 * Visits an {@link AnalyzeStm}.
	 */
	public void visitAnalyzeStm(AnalyzeStm s);
	
	/**
	 * Visits a {@link NodeStm}.
	 */
	public void visitNodeStm(NodeStm s);
	
	/**
	 * Visits a {@link ValidateStm}.
	 */
	public void visitValidateStm(ValidateStm s);
	
	/**
	 * Visits a {@link PlugStm}.
	 */
	public void visitPlugStm(PlugStm s);
	
	/**
	 * Visits a {@link GapifyStm}.
	 */
	public void visitGapifyStm(GapifyStm s);

	/**
	 * Visits an {@link InsertStm}.
	 */
	public void visitInsertStm(InsertStm s);
	
	/**
	 * Visits a {@link NopStm}.
	 */
	public void visitNopStm(NopStm s);

	public void visitArrayReadStm(ArrayReadStm s);

	public void visitArrayWriteStm(ArrayWriteStm s);

	public void visitEscapeStm(EscapeStm s);

	public void visitCallStm(CallStm s);

	public void visitArrayWriteStringStm(ArrayWriteStringStm s);
}
