package dk.brics.xact.analysis.soot;

import java.util.Arrays;

import soot.ValueBox;

public final class ArrayConstantInfo implements Cloneable {
	/**
	 * If null, nothing is known about the array (including its length).
	 * Otherwise, the array has same length as <tt>contents</tt>, and the
	 * entry at index <i>i</i> is
	 * <ul>
	 * <li>Same as <tt>contents[i]</tt> if <tt>contents[i]</tt> is not <tt>null</tt>
	 * <li>Uninitialized if <tt>contents[i]</tt> is <tt>null</tt>
	 * </ul>
	 */
	public final ValueBox[] contents;

	public ArrayConstantInfo(ValueBox[] contents) {
		this.contents = contents;
	}
	
	@Override
	public ArrayConstantInfo clone() {
		if (contents == null)
			return this;
		else
			return new ArrayConstantInfo(contents.clone());
	}
	
	public ArrayConstantInfo leastUpperBound(ArrayConstantInfo other) {
		// if either is TOP return TOP
		if (contents == null || other.contents == null)
			return new ArrayConstantInfo(null);
		if (Arrays.equals(contents, other.contents))
			return this;
		else
			return new ArrayConstantInfo(null);
	}
	
	public static final ArrayConstantInfo UNKNOWN = new ArrayConstantInfo(null);
}
