package dk.brics.xact.analysis.transformations;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dk.brics.xact.analysis.flowgraph.Edge;
import dk.brics.xact.analysis.flowgraph.FlowGraph;
import dk.brics.xact.analysis.flowgraph.Statement;
import dk.brics.xact.analysis.flowgraph.VariableFilter;

/**
 * Graphviz dot printer for flow graphs.
 */
public class FlowGraph2Dot {
	
	private PrintStream p;

	/**
	 * Constructs a new converter.
	 */
	public FlowGraph2Dot(PrintStream p) {
		this.p = p;
	}
	
	/**
	 * Prints the given flow graph in Graphviz dot format.
	 */
	public void print(FlowGraph g) {
		print(g, false);
	}
	public void print(FlowGraph g, boolean ignoreExits) {
		p.println("digraph FlowGraph {");
		printBody(g, ignoreExits, "");
		p.println("}");
		p.flush();
	}
	public void print(List<FlowGraph> graphs, boolean ignoreExits) {
		p.println("digraph FlowGraph {");
		int i=0;
		for (FlowGraph g : graphs) {
			i++;
			p.append(" subgraph clusterG").append("" + i).println(" {");
			p.append(" label=\"Subgraph ").append("" + i).print("\"");
			printBody(g, ignoreExits, "" + i);
			p.println("}");
		}
		p.println("}");
		p.flush();
	}
	public void printBody(FlowGraph g, boolean ignoreExits, String namesuffix) {
		Map<Statement,String> names = new HashMap<Statement,String>();
		// nodes
		int i = 0;
		for (Statement s : g.getNodes()) {
			String name = "S" + (++i) + "x" + namesuffix;
			names.put(s, name);
			p.append("  ").append(name)
			.append(" [label=\"").append(esc(s.toString()).replace(" at ", "\\n")).println("\", shape=box];");
		}
		// edges
		for (Statement s : g.getNodes())
			for (Edge<Statement,VariableFilter> e : g.getOutEdges(s))
				if (!ignoreExits || !e.getTo().toString().contains("method exit") || g.getOutEdges(s).size() == 1) {
					p.append("  ").append(names.get(e.getFrom())).append(" -> ").append(names.get(e.getTo()))
					.append(" [label=\"").append(esc(e.getData().toString())).println("\"];");
				}
		// entries
		p.append("  ROOT").append(namesuffix).append(" [label=\"\", shape=plaintext];\n");
		for (Statement s : g.getEntries())
			p.append("ROOT").append(namesuffix).append("-> ").append(names.get(s)).println(";");
	}
	
	private String esc(String s) {
		StringBuilder b = new StringBuilder();
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c >= 0x20 && c <= 0x7e && c!='\\' && c!='"')
				b.append(c);
			else {
				b.append("\\\\u");
				String t = Integer.toHexString(c & 0xffff);
				for (int j = 0; j + t.length() < 4; j++)
					b.append('0');
				b.append(t);
			}
		}
		return b.toString();
	}
}
