package dk.brics.xact.analysis.xmlgraph;

import java.util.HashMap;
import java.util.Map;

import dk.brics.xact.analysis.flowgraph.Statement;
import dk.brics.xact.analysis.flowgraph.TemplateConstant;
import dk.brics.xact.analysis.flowgraph.statements.ConcatStm;
import dk.brics.xact.analysis.flowgraph.statements.CopyStm;
import dk.brics.xact.analysis.flowgraph.statements.GapifyStm;
import dk.brics.xact.analysis.flowgraph.statements.GetStm;
import dk.brics.xact.analysis.flowgraph.statements.InsertStm;
import dk.brics.xact.analysis.flowgraph.statements.NodeStm;
import dk.brics.xact.analysis.flowgraph.statements.PlugStm;
import dk.brics.xact.analysis.flowgraph.statements.RemoveStm;
import dk.brics.xact.analysis.flowgraph.statements.SetStm;
import dk.brics.xact.analysis.flowgraph.statements.ValidateStm;
import dk.brics.xmlgraph.AttributeNode;
import dk.brics.xmlgraph.ChoiceNode;
import dk.brics.xmlgraph.ElementNode;
import dk.brics.xmlgraph.Node;
import dk.brics.xmlgraph.OneOrMoreNode;
import dk.brics.xmlgraph.SequenceNode;
import dk.brics.xmlgraph.TextNode;
import dk.brics.xmlgraph.XMLGraph;
import dk.brics.xmlgraph.XMLGraphFragment;

/**
 * Contains all the relationships between statements and their corresponding nodes in the XML graph.
 * These relationsships are established by {@link XMLGraphBuilder} and used by {@link XMLGraphAnalysis}.
 */
public final class StatementNodes {
	
	private Map<Statement, ChoiceNode> choices = new HashMap<Statement,ChoiceNode>();
	private Map<Statement, ChoiceNode> choices2 = new HashMap<Statement,ChoiceNode>();
	private Map<Statement, ChoiceNode> choices3 = new HashMap<Statement,ChoiceNode>();
	private Map<Statement, ChoiceNode> choices4 = new HashMap<Statement,ChoiceNode>();
	private Map<Statement, SequenceNode> seqs = new HashMap<Statement, SequenceNode>();
	private Map<Statement, TextNode> texts = new HashMap<Statement, TextNode>();
	private Map<Statement, OneOrMoreNode> oneormore = new HashMap<Statement, OneOrMoreNode>();
	private Map<Statement, ElementNode> elem = new HashMap<Statement, ElementNode>();
	private Map<Statement, AttributeNode> attr = new HashMap<Statement, AttributeNode>();
	private Map<Statement, Node> stmnodes = new HashMap<Statement, Node>();
	private Map<TemplateConstant, XMLGraphFragment> consts = new HashMap<TemplateConstant, XMLGraphFragment>();
	private Map<ValidateStm, Map<String,ChoiceNode>> validategaps = new HashMap<ValidateStm, Map<String,ChoiceNode>>();
	
	private SequenceNode emptySequenceNode;
	
	
	public SequenceNode getEmptySequenceNode(XMLGraph g) {
		return (SequenceNode)g.getNode(emptySequenceNode.getIndex());
	}
	public void setEmptySequenceNode(SequenceNode emptySequenceNode) {
		this.emptySequenceNode = emptySequenceNode;
	}
	
	///// set
	public TextNode getSetTextNode(SetStm s, XMLGraph g) {
		return (TextNode)g.getNode(texts.get(s).getIndex());
	}
	public void setSetTextNode(SetStm s, TextNode node) {
		texts.put(s, node);
	}
	
	///// insert
	public ChoiceNode getInsertLeftSide(InsertStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices.get(s).getIndex());
	}
	public ChoiceNode getInsertRightSide(InsertStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices2.get(s).getIndex());
	}
	public SequenceNode getInsertSequenceNode(InsertStm s, XMLGraph g) {
		return (SequenceNode)g.getNode(seqs.get(s).getIndex());
	}
	public TextNode getInsertTextNode(InsertStm s, XMLGraph g) {
		return (TextNode)g.getNode(texts.get(s).getIndex());
	}
	public void setInsertLeftSide(InsertStm s, ChoiceNode node) {
		choices.put(s, node);
	}
	public void setInsertRightSide(InsertStm s, ChoiceNode node) {
		choices2.put(s, node);
	}
	public void setInsertSequenceNode(InsertStm s, SequenceNode node) {
		seqs.put(s, node);
	}
	public void setInsertTextNode(InsertStm s, TextNode node) {
		texts.put(s, node);
	}
	
	///// concat
	public ChoiceNode getConcatChoiceNode(ConcatStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices.get(s).getIndex());
	}
	public TextNode getConcatTextNode(ConcatStm s, XMLGraph g) {
		return (TextNode)g.getNode(texts.get(s).getIndex());
	}
	public SequenceNode getConcatSequenceNode(ConcatStm s, XMLGraph g) {
		return (SequenceNode)g.getNode(seqs.get(s).getIndex());
	}
	public OneOrMoreNode getConcatOneOrMoreNode(ConcatStm s, XMLGraph g) {
		return (OneOrMoreNode)g.getNode(oneormore.get(s).getIndex());
	}
	public void setConcatChoiceNode(ConcatStm s, ChoiceNode node) {
		choices.put(s, node);
	}
	public void setConcatTextNode(ConcatStm s, TextNode node) {
		texts.put(s, node);
	}
	public void setConcatSequenceNode(ConcatStm s, SequenceNode node) {
		seqs.put(s, node);
	}
	public void setConcatOneOrMoreNode(ConcatStm s, OneOrMoreNode node) {
		oneormore.put(s, node);
	}
	
	///// gapify
	public ChoiceNode getGapifyChoiceNode(GapifyStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices.get(s).getIndex());
	}
	public void setGapifyChoiceNode(GapifyStm s, ChoiceNode node) {
		choices.put(s, node);
	}
	
	///// get
	public ChoiceNode getGetChoiceNode(GetStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices.get(s).getIndex());
	}
	public void setGetChoiceNode(GetStm s, ChoiceNode node) {
		choices.put(s, node);
	}
	public SequenceNode getGetEmptySequence(GetStm s, XMLGraph g) {
	    return (SequenceNode)g.getNode(seqs.get(s).getIndex());
	}
	public void setGetEmptySequence(GetStm s, SequenceNode node) {
	    seqs.put(s, node);
	}
	public OneOrMoreNode getGetOneOrMoreNode(GetStm s, XMLGraph g) {
		return (OneOrMoreNode)g.getNode(oneormore.get(s).getIndex());
	}
	public void setGetOneOrMoreNode(GetStm s, OneOrMoreNode node) {
		oneormore.put(s, node);
	}
	
	///// plug
	public TextNode getPlugTextNode(PlugStm s, XMLGraph g) {
		return (TextNode)g.getNode(texts.get(s).getIndex());
	}
	public SequenceNode getPlugSequenceNode(PlugStm s, XMLGraph g) {
		return (SequenceNode)g.getNode(seqs.get(s).getIndex());
	}
	public void setPlugTextNode(PlugStm s, TextNode node) {
		texts.put(s, node);
	}
	public void setPlugSequenceNode(PlugStm s, SequenceNode node) {
		seqs.put(s, node);
	}
	public ChoiceNode getPlugWrapTopNode(PlugStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices.get(s).getIndex());
	}
	public void setPlugWrapTopNode(PlugStm s, ChoiceNode node) {
		choices.put(s, node);
	}
	public ChoiceNode getPlugWrapContentNode(PlugStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices2.get(s).getIndex());
	}
	public void setPlugWrapContentNode(PlugStm s, ChoiceNode node) {
		choices2.put(s, node);
	}
	
	///// remove
	public SequenceNode getRemoveSequenceNode(RemoveStm s, XMLGraph g) {
		return (SequenceNode)g.getNode(seqs.get(s).getIndex());
	}
	public void setRemoveSequenceNode(RemoveStm s, SequenceNode node) {
		seqs.put(s, node);
	}
	
	///// template constant
	public XMLGraphFragment getTemplateConstantXMLFragment(TemplateConstant c) {
		return consts.get(c);
	}
	public void setTemplateConstantXMLFragment(TemplateConstant c, XMLGraphFragment f) {
		consts.put(c, f);
	}
	
	///// node stm
	public Node getStmNode(NodeStm s, XMLGraph g) {
		return g.getNode(stmnodes.get(s).getIndex());
	}
	void setStmNode(NodeStm s, Node node) {
		stmnodes.put(s, node);
	}
	public ChoiceNode getStmNextNode(NodeStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices.get(s).getIndex());
	}
	void setStmNextNode(NodeStm s, ChoiceNode node) {
		choices.put(s, node);
	}
	public ChoiceNode getStmFirstAttribute(NodeStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices2.get(s).getIndex());
	}
	void setStmFirstAttribute(NodeStm s, ChoiceNode node) {
		choices2.put(s, node);
	}
	public ChoiceNode getStmFirstChild(NodeStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices3.get(s).getIndex());
	}
	void setStmFirstChild(NodeStm s, ChoiceNode node) {
		choices3.put(s, node);
	}
	public ChoiceNode getStmGap(NodeStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices4.get(s).getIndex());
	}
	void setStmGap(NodeStm s, ChoiceNode node) {
		choices4.put(s, node);
	}
	
	///// copy stm
	public SequenceNode getCopyTopNode(CopyStm s, XMLGraph g) {
		return (SequenceNode)g.getNode(seqs.get(s).getIndex());
	}
	public void setCopyTopNode(CopyStm s, SequenceNode n) {
		seqs.put(s, n);
	}
	public ChoiceNode getCopyLeftChoice(CopyStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices4.get(s).getIndex());
	}
	public void setCopyLeftChoice(CopyStm s, ChoiceNode n) {
		choices4.put(s, n);
	}
	public ElementNode getCopyElementNode(CopyStm s, XMLGraph g) {
		return (ElementNode)g.getNode(elem.get(s).getIndex());
	}
	public void setCopyElementNode(CopyStm s, ElementNode n) {
		elem.put(s, n);
	}
	public ChoiceNode getCopyFirstAttribute(CopyStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices.get(s).getIndex());
	}
	public void setCopyFirstAttribute(CopyStm s, ChoiceNode n) {
		choices.put(s, n);
	}
	public ChoiceNode getCopyFirstChild(CopyStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices2.get(s).getIndex());
	}
	public void setCopyFirstChild(CopyStm s, ChoiceNode n) {
		choices2.put(s, n);
	}
	public ChoiceNode getCopyNextNode(CopyStm s, XMLGraph g) {
		return (ChoiceNode)g.getNode(choices3.get(s).getIndex());
	}
	public void setCopyNextNode(CopyStm s, ChoiceNode n) {
		choices3.put(s, n);
	}
	public AttributeNode getCopyAttributeNode(CopyStm s, XMLGraph g) {
		return (AttributeNode)g.getNode(attr.get(s).getIndex());
	}
	public void setCopyAttributeNode(CopyStm s, AttributeNode n) {
		attr.put(s, n);
	}
	public TextNode getCopyTextNode(CopyStm s, XMLGraph g) {
		return (TextNode)g.getNode(texts.get(s).getIndex());
	}
	public void setCopyTextNode(CopyStm s, TextNode n) {
		texts.put(s, n);
	}
	
	//// validate
	public ChoiceNode getValidateGap(ValidateStm s, String gapname, XMLGraph g) {
		return (ChoiceNode)g.getNode(validategaps.get(s).get(gapname).getIndex());
	}
	public void setValidateGap(ValidateStm s, String gapname, ChoiceNode node) {
		if (!validategaps.containsKey(s))
			validategaps.put(s, new HashMap<String, ChoiceNode>());
		validategaps.get(s).put(gapname, node);
	}
	
}
