package dk.brics.xact.analysis.xmlgraph;

import dk.brics.automaton.Automaton;
import dk.brics.automaton.Datatypes;
import dk.brics.misc.Origin;
import dk.brics.xact.XMLException;
import dk.brics.xact.analysis.flowgraph.SchemaType;

public class XMLSchemaDatatypes {
	
    public static final String NAMESPACE = "http://www.w3.org/2001/XMLSchema-datatypes";

    /**
     * Returns true if the specified type has the XMLSchema-datatypes namespace
     * prefix. This does not mean that it is actually a welldefined type.
     * @param schema schema type
     * @return boolean 
     */
    public static boolean isDatatype(SchemaType schema) {
        return schema.getType().startsWith("{" + NAMESPACE + "}");
    }
    
    /**
     * Returns true if the specified type has the XMLSchema-datatypes namespace
     * prefix. This does not mean that it is actually a welldefined type.
     * @param type expanded name
     * @return boolean 
     */
    public static boolean isDatatype(String type) {
        return type.startsWith("{" + NAMESPACE + "}");
    }
    
    public static Automaton getDatatype(SchemaType schema) throws XMLException {
        if (schema.getGapTypes() != null && schema.getGapTypes().size() > 0) {
            throw new XMLException("Datatypes cannot have gap types", schema.getOrigin());
        }
        return getDatatype(schema.getType(), schema.getOrigin());
    }
    
    public static Automaton getDatatype(String type, Origin origin) {
        String name = type.substring(NAMESPACE.length() + 2);
        if (!Datatypes.exists(name)) {
            throw new XMLException("No datatype is named " + name, origin);
        }
        return Datatypes.get(name);
    }
}
