package dk.brics.xact.wrappers;

import dk.brics.xact.Node;

/**
 * Base class for {@link Node} wrappers.
 */
abstract public class NodeWrapper<T extends Node> {

	private T real;
	
	private final ElementWrapper parent;

	/**
	 * Constructs a new wrapper.
	 */
	public NodeWrapper(T real, ElementWrapper parent) {
		this.real = real;
		this.parent = parent;
	}
	
	/**
	 * Sets the node being wrapped.
	 */
	public void setReal(T real) {
		this.real = real;
	}

	/**
	 * Returns the node being wrapped.
	 */
	public T getReal() {
		return real;
	}
	
	/**
	 * Returns the parent, null if at the root.
	 */
	public ElementWrapper getParent() {
		return parent;
	}
	
	/**
	 * Visits this wrapper by the given visitor. 
	 */
	abstract public void visitBy(WrapperNodeVisitor v);
}

