package dk.brics.xact.wrappers;

import dk.brics.xact.TempNode;
import dk.brics.xact.TemplateGap;
import dk.brics.xact.XML;

/**
 * Base class for {@link TempNode} wrappers.
 */
abstract public class TempNodeWrapper<T extends TempNode> extends NodeWrapper<T> {
	
	private TempNodeWrapper<? extends TempNode> nextsibling;
	
	private boolean touched_nextsibling;
	
	/**
	 * Constructs a new wrapper.
	 */
	public TempNodeWrapper(T real, ElementWrapper parent) {
		super(real, parent);
	}
	
	/**
	 * Returns the next wrapped XML node.
	 */
	public XML getRealNextSibling() {
		if (nextsibling != null)
			return nextsibling.getReal();
		else
			return getReal().getNextSibling();
	}
	
	/**
	 * Returns the next sibling node wrapper.
	 */
	public TempNodeWrapper<? extends TempNode> getNextSiblingWrapper() {
		return nextsibling;
	}
	
	/**
	 * Returns the next concrete sibling node wrapper.
	 */
	public ConcreteTempNodeWrapper<? extends TempNode> getNextSibling() {
		if (!touched_nextsibling) {
			nextsibling = make(getReal().getNextSibling(), getParent());
			touched_nextsibling = true;
		}
		TempNodeWrapper<? extends TempNode> x = nextsibling;
		if (x != null && !(x instanceof ConcreteTempNodeWrapper<?>))
			x = x.getNextSibling();
		return (ConcreteTempNodeWrapper<? extends TempNode>)x;
	}

	/**
	 * Makes a wrapper for the given XML node.
	 */
	public static TempNodeWrapper<? extends TempNode> make(XML x, ElementWrapper parent) {
		TempNodeWrapper<? extends TempNode> res;
		if (x == null)
			res = null;
		else if (x.isElement())
			res = new ElementWrapper(x.asElement(), parent);
		else if (x.isText())
			res = new TextWrapper(x.asText(), parent);
		else if (x.isComment())
			res = new CommentWrapper(x.asComment(), parent);
		else if (x.isProcessingInstruction())
			res = new ProcessingInstructionWrapper(x.asProcessingInstruction(), parent);
		else
			res = new TemplateGapWrapper((TemplateGap)x, parent);
		return res;
	}
}
