package dk.brics.xact.wrappers;

import dk.brics.xact.AttrNode;
import dk.brics.xact.Attribute;
import dk.brics.xact.AttributeGap;
import dk.brics.xact.Comment;
import dk.brics.xact.Element;
import dk.brics.xact.ProcessingInstruction;
import dk.brics.xact.TempNode;
import dk.brics.xact.TemplateGap;
import dk.brics.xact.Text;

/**
 * Abstract visitor for wrapper nodes.
 */
abstract public class WrapperNodeVisitor {

	/**
	 * Constructs a new visitor.
	 */
	public WrapperNodeVisitor() {}
	
	/**
	 * Visits an element node wrapper.
	 * Default implementation invokes {@link #visit(ConcreteTempNodeWrapper)}.
	 */
	public void visit(ElementWrapper w) {
		visit((ConcreteTempNodeWrapper<Element>)w);
	}
	
	/**
	 * Visits a text node wrapper.
	 * Default implementation invokes {@link #visit(ConcreteTempNodeWrapper)}.
	 */
	public void visit(TextWrapper w) {
		visit((ConcreteTempNodeWrapper<Text>)w);
	}
	
	/**
	 * Visits a comment node wrapper.
	 * Default implementation invokes {@link #visit(ConcreteTempNodeWrapper)}.
	 */
	public void visit(CommentWrapper w) {
		visit((ConcreteTempNodeWrapper<Comment>)w);
	}
	
	/**
	 * Visits a processing instruction node wrapper.
	 * Default implementation invokes {@link #visit(ConcreteTempNodeWrapper)}.
	 */
	public void visit(ProcessingInstructionWrapper w) {
		visit((ConcreteTempNodeWrapper<ProcessingInstruction>)w);
	}
	
	/**
	 * Visits a concrete template node wrapper.
	 * Default implementation invokes {@link #visit(TempNodeWrapper)}.
	 */
	public void visit(ConcreteTempNodeWrapper<?> w) {
		visit((TempNodeWrapper<?>)w);
	}
	
	/**
	 * Visits a template node wrapper.
	 * Default implementation does nothing.
	 */
	public void visit(TempNodeWrapper<? extends TempNode> w) {}
	
	/**
	 * Visits a template gap node wrapper.
	 * Default implementation invokes {@link #visit(TempNodeWrapper)}.
	 */
	public void visit(TemplateGapWrapper w) {
		visit((TempNodeWrapper<TemplateGap>)w);
	}
	
	/**
	 * Visits an attribute node wrapper.
	 * Default implementation invokes {@link #visit(AttrNodeWrapper)}.
	 */
	public void visit(AttributeWrapper w) {
		visit((AttrNodeWrapper<Attribute>)w);
	}
	
	/**
	 * Visits an attribute gap node wrapper.
	 * Default implementation invokes {@link #visit(AttrNodeWrapper)}.
	 */
	public void visit(AttributeGapWrapper w) {
		visit((AttrNodeWrapper<AttributeGap>)w);
	}
	
	/**
	 * Visits an attribute-like node wrapper.
	 * Default implementation does nothing.
	 */
	public void visit(AttrNodeWrapper<? extends AttrNode> w) {}
}
